/*
// $Id: JavaCUPTask.java$
// (C) Copyright 2001 Kana Software, Inc.
// jhyde, 7 February, 2002
*/

package java_cup;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import java.util.Vector;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.IOException;

/**
 * <code>JavaCUPTask</code> is an <a
 * href="http://jakarta.apache.org/ant/">ANT</a> task to invoke <a
 * href="http://www.cs.princeton.edu/~appel/modern/java/CUP/">JavaCUP</a>.
 *
 * Not all options are exposed yet.
 *
 * @author jhyde
 * @since 7 February, 2002
 * @version $Id$
 **/
public class JavaCUPTask extends Task {

	String parserClass;
	String symbolsClass;
	String inputFileName;
	File srcDir = new File(System.getProperty("user.dir"));
	Vector argVector = new Vector();

	public void execute() throws BuildException {
		try {
			String[] args = getArgs();
			Main.main(args, inputFileName);
		} catch (IOException e) {
			throw new BuildException(e.toString());
		} catch (internal_error e) {
			throw new BuildException(e.toString());
		} catch (Exception e) {
			throw new BuildException(e.toString());
		}
	}

	private String[] getArgs()
	{
		if (parserClass == null) {
			parserClass = Main.fileNameToClass(relativeFileName(inputFileName));
		}
		String packageName, className;
		int lastDot = parserClass.lastIndexOf(".");
		if (lastDot < 0) {
			packageName = null;
			className = parserClass;
		} else {
			packageName = parserClass.substring(0, lastDot);
			className = parserClass.substring(lastDot + 1);
		}
		if (packageName != null) {
			argVector.addElement("-package");
			argVector.addElement(packageName);
		}
		argVector.addElement("-parser");
		argVector.addElement(className);
		if (symbolsClass == null) {
			symbolsClass = className + "Sym";
		}
		argVector.addElement("-symbols");
		argVector.addElement(symbolsClass);
		if (inputFileName == null) {
			throw new BuildException("You must specify inputFile.");
		}
		String[] args = new String[argVector.size()];
		argVector.copyInto(args);
		return args;
	}

	/**
	 * Sets the name of the parser class will belong to, for example
	 * "com.mything.MyParser". By default, it is derived from the directory
	 * name of the input file: if the input is "com/mything/MyParser.cup", the
	 * class will be "com.mything.MyParser".
	 **/
	public void setParserClass(String parserClass) {
		this.parserClass = parserClass;
	}

	/**
	 * Sets the name of the symbols class. By default, this is the class name
	 * with "Sym" addElemented, for example "com.mything.MyParserSym".
	 **/
	public void setSymbolsClass(String symbolsClass) {
		this.symbolsClass = symbolsClass;
	}

	/** Sets <code>-dump</code>. **/
	public void setDump(boolean dump) {
		if (dump) {
			argVector.addElement("-dump");
		}
	}

	/** Sets <code>-interface</code>. **/
	public void setInterface(boolean _interface) {
		if (_interface) {
			argVector.addElement("-interface");
		}
	}

	/** Sets the name of the file from which to read the JavaCUP
	 * specification. **/
	public void setInput(File inputFile) {
		inputFileName = inputFile.toString();
	}

	/** Sets the name of the directory filenames are relative to. **/
	public void setSrcDir(File srcDir) {
		this.srcDir = srcDir;
		argVector.addElement("-srcdir");
		argVector.addElement(srcDir.toString());
	}

    /**
     * Makes a filename relative, by subtracting off the current directory, if
     * possible.
     **/
	String relativeFileName(String fileName)
    {
		char fileSep = System.getProperty("file.separator").charAt(0); // e.g. '/'
		String srcDirName = srcDir.toString() + fileSep; // e.g. 'E:/foo/'
		if (fileName.toUpperCase().startsWith(srcDirName.toUpperCase())) {
			fileName = fileName.substring(srcDirName.length());
		}
		return fileName;
    }
}


// End JavaCUPTask.java
